#include <iostream.h>

#include <aolconn.h>

AOLConn :: AOLConn(string &_hostname, int _port, string &_nickname, string &_password)
           : TCPThread("AOL Socket")
{
  hostname = _hostname;
  port     = _port;

  nickname = _nickname;
  password = _password;

  state    = DisconnectedS;
  docrc    = true;

  arch         = 9;
  versionMajor = 184;
  versionMinor = 1;
}

AOLConn :: ~AOLConn()
{
  Close();

  State(DisconnectedS);
}

void AOLConn :: State(int s)
{
  state = s;

  if (state == OnlineS)
    Online();
  else
  if (state == DisconnectedS)
    Close();
}

void AOLConn :: AddModule(AOLModule * module)
{
  modules.push_back(module);
}

void AOLConn :: RequestPrefix(string prefix, AOLModule * module)
{
  prefixes[prefix] = module;
}

bool AOLConn :: Connect()
{
  return (TCPThread::Connect(hostname, port));
}

bool AOLConn :: Connect(string &_hostname, int _port,
                          string _nickname = "", string _password = "")
{
  if (_nickname != "")
    nickname = _nickname;
  if (_password != "")
    password = _password;

  return (TCPThread::Connect(_hostname, _port));
}

void AOLConn :: Signoff()
{
}

void AOLConn :: Connected()
{
  State(ConnectedS);

  P3 * p3 = new P3(P3::InitT, Stringify(arch) +
                              Stringify(versionMajor) +
                              Stringify(versionMinor) +
                              ConnectStr);
  p3->Write(this);
  delete p3;

  //docrc = false;
}

void AOLConn :: Online()
{
}

void AOLConn :: DataWaiting()
{
  Buffer * headBuffer, * payloadBuffer;
  P3 * p3, * p32;

  if (!(headBuffer = Read(8)))
    return;

  p3 = new P3(headBuffer);
  payloadBuffer = Read(p3->length + 1);
  p3->Payload(payloadBuffer);

  beat ++;
  P3Ack = p3->sequence;

//  printf("internal: crc = %d length = %d sequence = %d ack = %d type = %d\n",
//         p3->crc, p3->length, p3->sequence, p3->ack, p3->type);


  cout << "internal: crc = " << p3->crc <<" length = " << p3->length <<" sequence = " << p3->sequence<<" ack =" <<" p3->ack"<<" type = " << p3->type <<"\n";
  
//printf("        : ");
//fmt_buf(p3->buffer->DataP(), p3->length);

  switch (state) {
    case ConnectedS:
      if (p3->type == P3::AckT) {
        string z = "44640001000100010a0101010b01010301"
                   + Stringify(nickname.length())
                   + Stringify(nickname)
                   + "011d00011d00010a01020301"
                   + Stringify(password.length())
                   + Stringify(password)
                   + "011d00000200";

        p32 = new P3(P3::DataT, z);
        p32->Write(this);
        delete p32;

        State(SignedonS);
      } else {
        State(DisconnectedS);
      }
      break;
    case SignedonS:
      if (p3->stringbuffer == "44332d2d2d2d") {
        p32 = new P3(P3::DataT, "4436");
        p32->Write(this);
        delete p32;

        State(OnlineS);
      }
      if (p3->stringbuffer == "647001") {
        State(DisconnectedS);
      }
      break;
    case OnlineS:
    {
      AOLModule * module;

      string code(p3->stringbuffer, 0, 4);
      p3->stringbuffer.erase(0, 4);
      p3->buffer->Skip(2);

      if ((module = prefixes[code])) {
        module->ReceiveP3(code, p3);
      } else
      if (p3->length >= 6 && code == "4154") {
        string id(p3->stringbuffer, 0, 4);
        p3->stringbuffer.erase(0, 4);
        p3->buffer->Skip(2);

        //cout << "a " << code << ": " << id << ": " << p3->stringbuffer << endl;
      } else
      if (p3->length >= 7 && code == "4174") {
        string id(p3->stringbuffer, 0, 6);
        p3->stringbuffer.erase(0, 6);
        p3->buffer->Skip(3);

        //cout << "a " << code << ": " << id << ": " << p3->stringbuffer << endl;
      } else
      if (code == "6174") {
        //cout << "a 6174: " << p3->stringbuffer << endl;
      } else
      if (p3->length >= 6) {
        string id(p3->stringbuffer, 0, 8);
        p3->stringbuffer.erase(0, 8);
        p3->buffer->Skip(4);

        //cout << "a " << code << ": " << id << ": " << p3->stringbuffer << endl;
      }
    }
      break;
    default:
      break;
  }

  if (p3->type == P3::HeartbeatT || (p3->type == P3::DataT && beat > 8)) {
    P3 Ack(P3::AckT);
    Ack.Write(this);
    beat = 0;
  }

  delete p3;
}

void AOLConn :: ReceiveP3(P3 * p3)
{
}

void AOLConn :: ReceiveFDO91()
{
}
