#ifndef _HAS_AOLCONN_H_
#define _HAS_AOLCONN_H_

#include <string>
#include <vector>
#include <hash_map>
#include <map>

#include <Byte.h>
#include <TCPThread.h>

#include <fdo91.h>
#include <p3.h>
#include <aolmodule.h>

class AOLModule;

class AOLConn : public TCPThread {
  friend class P3;

  public:
    AOLConn(string &_hostname, int _port,
              string &_nickname, string &_password);
    virtual ~AOLConn();

    void State(int s);
    enum States {
      DisconnectedS = 0,
      ConnectedS,
      SignedonS,
      OnlineS
    };

    void AddModule(AOLModule * module);
    void RequestPrefix(string prefix, AOLModule * module);

    bool Connect();
    bool Connect(string &_hostname, int _port,
                 string _nickname = "", string _password = "");
    void Signoff();

    virtual void Connected();
    virtual void DataWaiting();
    virtual void ReceiveP3(P3 * p3);
    virtual void ReceiveFDO91();

    virtual void Online();

  protected:
    string hostname;
    int port;
    string nickname, password;

    byte arch, versionMajor, versionMinor;

    int P3Seq, P3Ack;
    int beat;

    byte state;
    bool docrc;

    hash_map<string, FDO91 *> fdo91s;
    vector<AOLModule *> modules;

    struct eqstr {
      bool operator() (const char * s1, const char * s2) const {
        return (strcmp(s1, s2) == 0);
      }
    };
    hash_map<const char *, AOLModule *, hash<const char *>, eqstr> p3s;
    map<string, AOLModule *> prefixes;
};

const string ConnectStr = "00000000000000000000000000000011000000000000000000000000000000000000000000000000000000000000";

#endif /* _HAS_P3_H_ */
