#include <aolip.h>

AOLip :: AOLip(AOLConn * _aolConn)
         : AOLModule(_aolConn)
{
  aolConn->AddModule(this);
  aolConn->RequestPrefix("7961", this);
  aolConn->RequestPrefix("7963", this);

  tap   = NULL;
  seqno = 0;
}

AOLip :: ~AOLip()
{
#if defined(linux)
  if (tap)
    delete tap;
#endif
}

void AOLip :: RequestIP()
{
  string z = "7961";

  P3 p3(P3::DataT, z);
  p3.Write(aolConn);
}

void AOLip :: ReceiveP3(string code, P3 * p3)
{
  if (code == "7961")
      Setup(p3);
  else
  if (code == "7963")
      Data(p3);
}

void AOLip :: Setup(P3 * p3)
{
  TLV tlv(p3->buffer);

  hostname          = tlv.GetString(10);
  local_addr.s_addr = ntohl(tlv.GetUInt32(3));
  dns_addr.s_addr   = ntohl(tlv.GetUInt32(4));

  printf("* aolip tunnel established, hostname = %s ", hostname.c_str());
  printf("address = %s ", inet_ntoa(local_addr));
  printf("dns address = %s\n", inet_ntoa(dns_addr));

#if defined(linux)
  tap = new aolTap("/dev/tap0", this);
  tap->Start();
#endif
}

void AOLip :: Data(P3 * p3)
{
  Datagram * dgram = new Datagram(p3->buffer);

  Input(dgram);

  delete dgram;
}

void AOLip :: Output(Datagram * dgram)
{
  Output(dgram->sequence, dgram->length, dgram->payload);
}

void AOLip :: Output(int sequence, int length, byte * payload)
{
  string z = "7963";
  P3 p3(P3::DataT);

  p3.buffer->PutRaw(z);
  p3.buffer->PutByte(sequence);
  p3.buffer->PutByte(length);
  p3.buffer->PutRaw(payload, length);
  p3.Write(aolConn);

  //printf("SENDING: ");
  //fmt_buf(p3.buffer->DataP(), p3.buffer->Length());
}

void AOLip :: Input(Datagram * dgram)
{
#if defined(linux)
  if (tap)
    tap->Output(dgram->payload, dgram->length);

  seqno = dgram->sequence;
#else
  cout << "* dummy IP input function" << endl;
#endif
}
