#ifndef _HAS_AOLIP_H_
#define _HAS_AOLIP_H_

#include <aolmodule.h>
#include <TLV.h>
#include <ethertap.h>

class AOLip : public AOLModule {
  public:
    AOLip(AOLConn * aolConn);
    virtual ~AOLip();

    void RequestIP();
    virtual void ReceiveP3(string code, P3 * p3);

    int Sequence() { return (seqno); }

    class Datagram;

    void Output(Datagram * dgram);
    void Output(int sequence, int length, byte * payload);
    void Input(Datagram * dgram);

  private:
    void Setup(P3 * p3);
    void Data(P3 * p3);

    int seqno;
    void Ack(int sequence);

    struct in_addr local_addr, dns_addr;
    string hostname;

    class Datagram {
      public:
        Datagram(Buffer * buffer) {
          sequence = buffer->GetByte();
          length   = buffer->GetByte();
          if (length & 128) {
            buffer->Rewind(1);
            length = buffer->GetUInt16() & 0x0fff;
          }
          payload  = buffer->GetRaw(length);
        }

        ~Datagram() {
          sequence = 0;
          length   = 0;
          Free(payload);
        }

        unsigned sequence, length;
        byte * payload;
    };

#if defined(linux)
    class aolTap : public Ethertap {
      public:
        aolTap(string _filename, AOLip * _aolIp) : Ethertap(_filename) { aolIp = _aolIp; }

        virtual void Input(byte * payload, int length) {
          aolIp->Output(aolIp->Sequence(), length, payload);
        };

      private:
        AOLip * aolIp;
    };
#endif

#if defined(linux)
    aolTap * tap;
#endif
};

#endif /* _HAS_AOLIP_H_ */
