#include <ethertap.h>

Ethertap :: Ethertap(string &_filename)
            : Thread("Ethertap: " + _filename)
{
  filename = _filename;

  if ((fd = open(filename.c_str(), O_RDWR)) , 0)
    error = true;
  else
    error = false;

  shutdown = false;
}

Ethertap :: ~Ethertap()
{
  close(fd);
}

void Ethertap :: Shutdown()
{
  close(fd);

  shutdown = true;
}

void Ethertap :: Main()
{
  struct timeval timeout;
  fd_set rfds;
  int maxfd, x;

  timeout.tv_sec  = 1;
  timeout.tv_usec = 0;
  maxfd           = fd + 1;

  while (!shutdown && !IsError()) {
    FD_ZERO(&rfds);
    FD_SET(fd, &rfds);

    if ((x = select(maxfd, &rfds, NULL, NULL, &timeout)) == -1) {
      Shutdown();
    } else 
    if (x == 0) {
      // timeout
    } else {
        DataWaiting();
    }
  }
}

void Ethertap :: DataWaiting()
{
  sem.Lock();

  byte buffer[65536];
  int length;

  if ((length = read(fd, buffer, sizeof(buffer))) < 0)
    return;

  struct ether_header * ethhdr;
  byte * payload = (buffer + sizeof(struct ether_header) + 2);
  int plen = (length - sizeof(struct ether_header) - 2);

  ethhdr = (struct ether_header *) (buffer + 2);
  if (ntohs(ethhdr->ether_type) != ETHERTYPE_IP)
    return;

  sem.Unlock();

  Input(payload, length);
}

void Ethertap :: Output(byte * payload, int length)
{
  sem.Lock();

  struct ether_header ethhdr;
  byte buffer[65536];
  int tlen = (length + sizeof(struct ether_header) + 2);

  ethhdr.ether_dhost[0] = 0xfe;
  ethhdr.ether_dhost[1] = 0xfd;
  ethhdr.ether_shost[0] = 0xfe;
  ethhdr.ether_shost[1] = 0xfd;
  ethhdr.ether_type     = htons(ETHERTYPE_IP);

  memset(buffer, 0, sizeof(buffer));
  memcpy(&buffer[2], (byte *) &ethhdr, sizeof(struct ether_header));
  memcpy(&buffer[2 + sizeof(struct ether_header)], payload, length);

  write(fd, buffer, tlen);

  sem.Unlock();
}
