#include <iostream>
#include <fdo91.h>

/****/

Atom :: Atom()
{
  major  = 0;
  minor  = 0;
  value  = (Buffer *) NULL;
}

Atom :: Atom(int _major, int _minor, int _length, byte * _value)
{
  major  = _major;
  minor  = _minor;
  length = _length;
  value  = new Buffer(_value, _length);
}

Atom :: Atom(int _major, int _minor, int _length, Buffer * _value)
{ 
  major  = _major;
  minor  = _minor;
  length = _length;
  value  = (Buffer *) _value;
}

Atom :: ~Atom()
{
}

int Atom :: GetNumber()
{
  int i = 0;

  if (length <= 0)
    return (i);

  if (length > 0)
    i = Value()[0] & 0xff;
  if (length > 1)
    i = i << 8 | Value()[1] & 0xff;
  if (length > 2)
    i = i << 8 | Value()[2] & 0xff;
  if (length > 3)
    i = i << 8 | Value()[3] & 0xff;

  return (i);
}

string Atom :: GetString()
{
  char * s;
  string z;

  s = (char *) Alloc(length + 1);
  memcpy(s, Value(), length);
  s[length] = '\0';

  z = s;
  Free(s);

  return (z);
}

/****/

FDO91 :: FDO91()
{
  id            = 0;
  lastmajor     = -1;
  majorHighBits = 0;
  minorHighBits = 0;
  buffer        = new Buffer(65536);

  buffer->Block(true);
}

FDO91 :: FDO91(string code, string number)
{
  id            = 0;
  lastmajor     = -1;
  majorHighBits = 0;
  minorHighBits = 0;
  buffer        = new Buffer(65536);

  buffer->PutRaw(code);
  buffer->PutRaw(number);
}

FDO91 :: FDO91(Buffer * _buffer)
{
  id            = 0;
  lastmajor     = -1;
  majorHighBits = 0;
  minorHighBits = 0;
  buffer        = (Buffer *) _buffer;

  buffer->Block(true);
}

FDO91 :: ~FDO91()
{
  if (buffer)
    delete buffer;
}

Atom * FDO91 :: GetAtom()
{
  Atom * atom;
  int v1, v2, v3, v4, v5, v6;
  int major, minor, length;
  byte * value;

  if (buffer->Available() <= 0)
    return (NULL);

  v1 = v2 = v3 = v4 = v5 = v6 = 0;
  major = minor = length = 0;
  value = (byte *) NULL;

  v3 = buffer->GetByte();
  v4 = (v3 >> 5);

  if (v4 == 7) {
    v2 = (v3 & 0x18) << 2;
    v1 = (v3 & 0x60) << 4;

    if ((v3 & 0x10) == 1) {
      majorHighBits = v2;
      minorHighBits = v1;
    }

    v3 = buffer->GetByte();
    v4 = (v3 >> 5);
  } else {
    v1 = minorHighBits;
    v2 = majorHighBits;
  }

  switch (v4) {
    case 0:
      major = v3 & 0x1f | v2;
      minor = buffer->GetByte();
      v5    = buffer->GetByte();

      if ((v5 >> 7) == 0) {
        length = v6;
      } else {
        v4     = buffer->GetByte();
        length = (v4 & 0x7f) << 7 | v4 & 0xff;
      }

      if (length > 0)
        value  = buffer->GetRaw(length);

      break;
    case 1:
      major   = v3 & 0x1f | v2;
      v4      = buffer->GetByte();
      length  = v4 >> 5;
      minor   = v4 & 0x1f | v1;

      if (length > 0)
        value = buffer->GetRaw(length);

      break;
    case 2:
      major   = v3 & 0x1f | v2;
      v4      = buffer->GetByte();
      minor   = v4 & 0x1f | v1;
      length  = 1;

      break;
    case 3:
      major   = lastmajor;
      minor   = v3 & 0x1f | v1;
      length  = 0;

      break;
    case 4:
      major   = lastmajor;
      minor   = v3 & 0x1f | v1;
      length  = buffer->GetByte();

      if (length > 0)
        value = buffer->GetRaw(length);

      break;
    case 5:
      major    = lastmajor;
      minor    = v3 & 0x1f | v1;
      length   = 1;
      value    = (byte *) Alloc(1);
      value[0] = 0;

      break;
    case 6:
      major    = lastmajor;
      minor    = v3 & 0x1f | v1;
      length   = 1;
      value    = (byte *) Alloc(1);
      value[1] = 1;

      break;
  }

  lastmajor = major;

  if (major == 0 && minor == 1) {
    cout << "  * Begin stream" << endl;
    return (GetAtom());
  }

  if (major == 0 && minor == 2) {
    cout << "  * End stream" << endl;
    return (GetAtom());
  }

  if (major == 0 && minor == 4 && length >= 3) {
    int length1;
    Atom * atom1;
    bool done = false;

    major   = value[0];
    minor   = value[1];
    length  = value[2];
    length1 = 0;

    cout << "  * Begin Large atom " << " major = " << major << " minor = " << minor << " length = " << length << endl;

    Free(value);
    value = (byte *) Alloc(length);

    while (!done) {
      if (!(atom1 = GetAtom())) {
        Free(value);
        return (NULL);
      }

      if (atom1->Major() == 0 && (atom1->Minor() == 5 || atom1->Minor() == 6)) {
        cout << "  * Atom segment " << atom1->Length() << " big" << endl;
        memcpy(&value[length1], atom1->Value(), atom1->Length());
        length1 += atom1->Length();
      }

      if (atom1->Major() == 0 && atom1->Minor() == 6) {
        cout << "  * End Large atom" << endl;
        done = true;
      }

      delete atom1;
    }
  }

  atom = new Atom(major, minor, length, value);

  cout << "Atom " << major << "." << minor << " " << length << endl;

  return(atom);
}

void FDO91 :: PutNumber(int major, int minor, int number, int flag)
{
  int k = major;
  int l = minor;
  unsigned long l1;
  byte c;

  l1 = number & 0xffffffff;

  if (flag == 1)
    c = 0;
  else
  if (flag == 4)
    c = 1;
  else
  if (lastmajor == k && l1 == 0 && l < 128)
    c = 160;
  else
  if (lastmajor == k && l1 == 1 && l < 128)
    c = 192;
  else
  if (l1 < 8 && l < 128)
    c = '@';
  else
  if (l < 128)
    c = ' ';
  else
    c = 0;

  switch (c) {
    case 0:
      if (k >= 32)
        buffer->PutByte(0xe0 | (k & 0x60) >> 2);

      buffer->PutByte(k & 0x1f);
      buffer->PutByte(l & 0xff);

      if (l1 == 0) {
        buffer->PutByte(0);
      } else
      if (l1 < 255) {
        buffer->PutByte(1);
        buffer->PutByte(l1);
      } else
      if (l1 < 65535) {
        buffer->PutByte(2);
        buffer->PutByte(l1 >> 8 & 255);
        buffer->PutByte(l1 & 255);
      } else
      if (l1 < 0xffffff) {
        buffer->PutByte(3);
        buffer->PutByte(l1 >> 16 & 255);
        buffer->PutByte(l1 >> 8 & 255);
        buffer->PutByte(l1 & 255);
      } else {
        buffer->PutByte(4);
        buffer->PutByte(l1 >> 24 & 255);
        buffer->PutByte(l1 >> 16 & 255);
        buffer->PutByte(l1 >> 8 & 255);
        buffer->PutByte(l1 & 255);
      }

      break;
    case 1:
      buffer->PutByte(k & 0x1f);
      buffer->PutByte(l & 0xff);

      buffer->PutByte(4);
      buffer->PutByte(l1 >> 24 & 255);
      buffer->PutByte(l1 >> 16 & 255);
      buffer->PutByte(l1 >> 8 & 255);
      buffer->PutByte(l1 & 255);

      break;
    case 32:
      if (k >= 32 || l >= 32)
        buffer->PutByte(0xe0 | (k & 0x60) >> 2 | (l & 0x60) >> 4);

      buffer->PutByte(0x20 | k & 0x1f);

      if (l1 == 0) {
        buffer->PutByte(l & 0x1f);
      } else
      if (l1 < 255) {
        buffer->PutByte(0x20 | l & 0x1f);
        buffer->PutByte(l1);
      } else
      if (l1 < 65535) {
        buffer->PutByte(0x40 | l & 0x1f);
        buffer->PutByte(l1 >> 7 & 255);
        buffer->PutByte(l1 & 255);
      } else
      if (l1 < 0xffffff) {
        buffer->PutByte(0x60 | l & 0x1f);
        buffer->PutByte(l1 >> 16 & 255);
        buffer->PutByte(l1 >> 8 & 255);
        buffer->PutByte(l1 & 255);
      } else {
        buffer->PutByte(0x80 | l & 0x1f);
        buffer->PutByte(l1 >> 24 & 255);
        buffer->PutByte(l1 >> 16 & 255);
        buffer->PutByte(l1 >> 8 & 255);
        buffer->PutByte(l1 & 255);
      }

      break;
    case 64:
      if (l >= 32)
        buffer->PutByte(0xe0 | (k & 0x60) >> 2 | (l & 0x60) >> 4);

      buffer->PutByte(0x40 | k & 0x1f);
      buffer->PutByte((l1 & 7) << 5 | (l & 0x1f));

      break;
    case 160:
      if (l >= 32)
        buffer->PutByte(0xe0 | (l & 0x60) >> 4);

      buffer->PutByte(0xa0 | l & 0x60);

      break;
    case 192:
      if (l >= 32)
        buffer->PutByte(0xe0 | (l & 0x60) >> 4);

      buffer->PutByte(0xc0 | l & 0x60);

      break;
    default:
      break;
  }
}

void FDO91 :: PutAtom(Atom * atom, int flag = 1)
{
  int j = atom->Major();
  int k = atom->Minor();
  int l = atom->Length();
  byte c;

  if (flag)
    c = 0;
  else
  if (lastmajor == j && l == 0 && k < 128)
    c = 96;
  else
  if (lastmajor == j && l < 256 && k > 128)
    c = 128;
  else
  if (l < 8)
    c = 32;

  switch (c) {
    case 0:
      if (j >= 32)
        buffer->PutByte(0xe0 | (j & 0x60) >> 2);

      buffer->PutByte(j & 0x1f);
      buffer->PutByte(k & 0xff);
      buffer->PutByte(l & 0x7f);
      buffer->PutRaw(atom->Value(), l);

      break;
    case 32:
      if (j >= 32 || k >= 32)
        buffer->PutByte(0xe0 | (j & 0x60) >> 2 | (k & 0x60) >> 4);

      buffer->PutByte(0x20 | j & 0x1f);
      buffer->PutByte((l & 0x70) << 5 | k & 0x1f);
      buffer->PutRaw(atom->Value(), l);

      break;
    case 96:
      if (k >= 32)
        buffer->PutByte(0xe0 | (k & 0x60) >> 4);

      buffer->PutByte(0x60 | k & 0x1f);

      break;
    case 128:
      if (k >= 32)
        buffer->PutByte(0xe0 | (k & 0x60) >> 4);

      buffer->PutByte(0x80 | k & 0x1f);
      buffer->PutByte(l & 0xff);
      buffer->PutRaw(atom->Value(), l);

      break;
    default:
      break;
  }
}

#ifdef FDO_MAIN
int main(int argc, char ** argv)
{
  Buffer * buffer;
  FDO91 * fdo91;
  Atom * atom;
  byte * data;
  string z1, z2;

  z1 = string(argv[1]);
  data = Byteify(z1);

  printf("len == %d\n", z1.length() / 2);

  buffer = new Buffer(data, z1.length()/2);
  fdo91 = new FDO91(buffer);

  while ((atom = fdo91->GetAtom()) != NULL) {
    cout << "  * Atom major = " << atom->Major() << " minor = " << atom->Minor() << " length = " << atom->Length() << endl;
    fmt_buf(atom->Value(), atom->Length());
    delete atom;
  }

  delete fdo91;
  Free(data);
  cout << "Done." << endl;
}
#endif
