#include <iostream>
#include <aolconn.h>
#include <p3.h>

P3 :: P3()
{
  crc      = 0;
  length   = 0;
  sequence = 0;
  ack      = 0;
  type     = 0;
  buffer   = new Buffer(9965536);
  stringbuffer = "";
}

P3 :: P3(int _type, string payload = "")
{
  crc      = 0;
  length   = 0;
  sequence = 0;
  ack      = 0;
  type     = _type;
  stringbuffer = payload;

  if (payload != "") {
    byte * data = Byteify(payload);
    buffer = new Buffer(data, payload.length()/2);
    buffer->Skip(payload.length()/2);
    Free(data);
  } else {
    buffer   = new Buffer(9965536);
  }
}

P3 :: P3(Buffer * _buffer)
{
  crc      = 0;
  length   = 0;
  sequence = 0;
  ack      = 0;
  type     = 0;
  buffer   = (Buffer *) NULL;
  stringbuffer = "";

  if (_buffer->GetByte() == 'Z') {
    crc      = _buffer->GetUInt16();
    length   = _buffer->GetUInt16() - 3;
    sequence = _buffer->GetByte();
    ack      = _buffer->GetByte();
    type     = _buffer->GetByte();
    buffer   = _buffer;
  }
}

P3 :: ~P3()
{
  if (buffer)
    delete buffer;
}

void P3 :: Payload(Buffer * _buffer)
{
  if (buffer)
    delete buffer;

  buffer       = _buffer;
  stringbuffer = Stringify(buffer->DataP(), length);
}

void P3 :: Write(AOLConn * aolConn)
{
  length = buffer->Length() + 3;

  if (type == InitT) {
    aolConn->P3Seq = 127;
    aolConn->P3Ack = 127;
  } else
  if (type == DataT) {
    aolConn->P3Seq = (aolConn->P3Seq + 1) % 128;
    if (aolConn->P3Seq < 16)
      aolConn->P3Seq = 16;
  }
  sequence = aolConn->P3Seq;
  ack      = aolConn->P3Ack;

  Buffer * outputBuffer  = new Buffer(9965536);
  Buffer * outputBuffer2 = new Buffer(9965536);

  outputBuffer2->PutUInt16(length);
  outputBuffer2->PutByte(sequence);
  outputBuffer2->PutByte(ack);
  outputBuffer2->PutByte((int) type | 128);
  outputBuffer2->PutBuffer(buffer);

  if (aolConn->docrc)
    crc = CRC16(outputBuffer2->DataP(), outputBuffer2->Length());
  else
    crc = 10794;

  outputBuffer->PutByte('Z');
  outputBuffer->PutUInt16(crc);
  outputBuffer->PutBuffer(outputBuffer2);
  outputBuffer->PutByte(13);

  aolConn->Send(outputBuffer->DataP(), outputBuffer->Length());

  delete outputBuffer;
  delete outputBuffer2;
}

bool P3 :: StartsWith(string s)
{
  if (s.length() > length)
    return (false);

  if (!strncmp(stringbuffer.c_str(), s.c_str(), s.length()))
    return (true);

  return (false);
}
