#include <Buffer.h>
#include <Memory.h>

#include <netinet/in.h>

Buffer :: Buffer(byte * data, int len, bool swap_ = true)
{
  if (!data)
    buffer = (byte *) Alloc(len);
  else
    buffer = (byte *) Copy(data, len);

  p    = buffer;
  pos  = 0;
  end  = len;
  swap = swap_;
  block= false;
}

Buffer :: Buffer(int len, bool swap_ = true)
{
  buffer = (byte *) Alloc(len);

  p    = buffer;
  pos  = 0;
  end  = len;
  swap = swap_;
  block= false;
}

Buffer :: Buffer(string str, bool swap_ = true)
{
  int len = str.length();
  char * data = (char *) str.data();

  buffer = (byte *) Copy(data, len);

  p    = buffer;
  pos  = 0;
  end  = len;
  swap = swap_;
  block= false;
}

Buffer :: Buffer(Buffer * buffer_)
{
  buffer = (byte *) Copy(buffer_->buffer, buffer_->end);

  p    = buffer;
  pos  = buffer_->pos;
  end  = buffer_->end;
  swap = buffer_->swap;
  block= false;
}

Buffer :: ~Buffer()
{
  if (buffer)
    Free(buffer);

  p    = NULL;
  pos  = 0;
  end  = 0;
  swap = false;
  block= false;
}

void Buffer :: Append(byte * data, int len)
{
  sem.Lock();

  int _length = Available();
  byte * b = (byte *) Alloc(_length + len);

  memcpy(b, p, _length);
  memcpy(&b[_length], data, len);

  pos = 0;
  end = _length + len;

  Free(buffer);
  buffer = b;
  p      = b;

  sem.Unlock();
}

void Buffer :: Block(bool b)
{
  block = b;
}

void Buffer :: Reset()
{
  p   = buffer;
  pos = 0;
}

int Buffer :: Length()
{
  return (pos);
}

int Buffer :: Available()
{
  return (end - pos);
}

int Buffer :: Total()
{
  return (end);
}

byte * Buffer :: Data()
{
  byte * rv;

  rv = (byte *) Copy(buffer, pos);

  return (rv);
}

byte * Buffer :: DataP() 
{
  return (buffer);
}

UInt8 Buffer :: GetUInt8()
{
  UInt8 rv;

  if ((pos + SizeofInt8) > end)
    return ((UInt8) -1);

  memcpy((byte *) &rv, p, SizeofInt8);
  p   += SizeofInt8;
  pos += SizeofInt8;

  return (rv);
}

int Buffer :: PutUInt8(UInt8 val)
{
  if ((pos + SizeofInt8) > end)
    return (-1);

  memcpy(p, (byte *) &val, SizeofInt8);
  p   += SizeofInt8;
  pos += SizeofInt8;

  return (1);
}

Int8 Buffer :: GetInt8()
{
  Int8 rv;

  if ((pos + SizeofInt8) > end)
    return ((Int8) -1);

  memcpy((byte *) &rv, p, SizeofInt8);
  p   += SizeofInt8;
  pos += SizeofInt8;

  return (rv);
}

int Buffer :: PutInt8(Int8 val)
{
  if ((pos + SizeofInt8) > end)
    return (-1);

  memcpy(p, (byte *) &val, SizeofInt8);
  p   += SizeofInt8;
  pos += SizeofInt8;

  return (1);
}

UInt16 Buffer :: GetUInt16()
{
  UInt16 rv;

  if ((pos + SizeofInt16) > end)
    return ((UInt16) -1);

  memcpy((byte *) &rv, p, SizeofInt16);
  p   += SizeofInt16;
  pos += SizeofInt16;

  if (swap)
    rv = ntohs(rv);

  return (rv);
}

int Buffer :: PutUInt16(UInt16 val)
{
  UInt16 ival = swap ? htons(val) : val;

  if ((pos + SizeofInt16) > end)
    return (-1);

  memcpy(p, (byte *) &ival, SizeofInt16);
  p   += SizeofInt16;
  pos += SizeofInt16;

  return (1);
}

Int16 Buffer :: GetInt16()
{
  Int16 rv;

  if ((pos + SizeofInt16) > end)
    return ((Int16) -1);

  memcpy((byte *) &rv, p, SizeofInt16);
  p   += SizeofInt16;
  pos += SizeofInt16;

  if (swap)
    rv = ntohs(rv);

  return (rv);
}

int Buffer :: PutInt16(Int16 val)
{
  Int16 ival = swap ? htons(val) : val;

  if ((pos + SizeofInt16) > end)
    return (-1);

  memcpy(p, (byte *) &ival, SizeofInt16);
  p   += SizeofInt16;
  pos += SizeofInt16;

  return (1);
}

UInt32 Buffer :: GetUInt32()
{
  UInt32 rv;

  if ((pos + SizeofInt32) > end)
    return ((UInt32) -1);

  memcpy((byte *) &rv, p, SizeofInt32);
  p   += SizeofInt32;
  pos += SizeofInt32;

  if (swap)
    rv = ntohl(rv);

  return (rv);
}

int Buffer :: PutUInt32(UInt32 val)
{
  UInt32 ival = swap ? htonl(val) : val;

  if ((pos + SizeofInt32) > end)
    return (-1);

  memcpy(p, (byte *) &ival, SizeofInt32);
  p   += SizeofInt32;
  pos += SizeofInt32;

  return (1);
}

Int32 Buffer :: GetInt32()
{
  Int32 rv;

  if ((pos + SizeofInt32) > end)
    return ((Int32) -1);

  memcpy((byte *) &rv, p, SizeofInt32);
  p   += SizeofInt32;
  pos += SizeofInt32;

  if (swap)
    rv = ntohl(rv);

  return (rv);
}

int Buffer :: PutInt32(Int32 val)
{
  Int32 ival = swap ? htonl(val) : val;

  if ((pos + SizeofInt32) > end)
    return (-1);

  memcpy(p, (byte *) &ival, SizeofInt32);
  p   += SizeofInt32;
  pos += SizeofInt32;

  return (1);
}

byte Buffer :: GetByte()
{
  byte rv;

  if ((pos + SizeofByte) > end)
    return ((byte) -1);

  memcpy((byte *) &rv, p, SizeofByte);
  p   += SizeofByte;
  pos += SizeofByte;

  return (rv);
}

int Buffer :: PutByte(byte val)
{
  if ((pos + SizeofByte) > end)
    return (-1);

  memcpy(p, (byte *) &val, SizeofByte);
  p   += SizeofByte;
  pos += SizeofByte;

  return (1);
}

byte * Buffer :: GetRaw(int len)
{
  byte * rv;

  if ((pos + len) > end || len < 0)
    return (NULL);

  rv = (byte *) Copy(p, len);

  p   += len;
  pos += len;

  return (rv);
}

int Buffer :: PutRaw(byte * data, int len)
{
  if ((pos + len) > end || len < 0)
    return (-1);

  memcpy(p, data, len);

  p   += len;
  pos += len;

  return (1);
}

int Buffer :: PutRaw(string &s)
{
  int i;
  byte * data;

  data = Byteify(s);
  i = PutRaw(data, s.length()/2);
  Free(data);

  return (i);
}

string Buffer :: GetString(int len)
{
  char * data;
  string rv;

  if ((pos + len) > end || len < 0)
    return ("");

  data = (char *) Alloc(len + 1);
  memcpy((byte *) data, p, len);

  p   += len;
  pos += len;
  rv   = data;

  Free(data);

  return (rv);
}

int Buffer :: PutString(string str)
{
  int len = str.length();
  char * data = (char *) str.data();

  if ((pos + len) > end)
    return (-1);

  memcpy(p, data, len);

  p   += len;
  pos += len;

  return (1);
}

string Buffer :: GetStringLen()
{
  int len;
  string rv;

  len = GetUInt8();
  rv = GetString(len);

  return (rv);
}

int Buffer :: PutStringLen(string str)
{
  int len = str.length();
  char * data = (char *) str.data();

  PutUInt8(len);

  if ((pos + len) > end)
    return (-1);

  memcpy(p, data, len);

  p   += len;
  pos += len;

  return (1);
}

int Buffer :: PutBuffer(Buffer * abuffer)
{
  int length = abuffer->Length();

  if ((pos + length) > end)
    return (-1);

  PutRaw(abuffer->DataP(), length);

  return (1);
}

Buffer * Buffer :: GetBuffer(int len)
{
  Buffer * rv;

  if ((pos + len) > end || len < 0)
    return (NULL);

  rv = new Buffer(p, len);

  return (rv);
}

int Buffer :: Skip(int len)
{
  if ((pos + len) > end || len < 0)
    return (-1);

  p   += len;
  pos += len;

  return (1);
}

int Buffer :: Rewind(int len)
{
  if ((pos - len) < 0)
    return (-1);

  p   -= len;
  pos -= len;

  return (1);
}
