#include <iostream>
#include <string>
#include <vector>

#include <ConfigFile.h>

ConfigEntry :: ConfigEntry()
{
}

ConfigEntry :: ConfigEntry(const string &_name, const string &_value)
{
  name  = _name;
  value = _value;
}

void ConfigEntry :: set(const string &_value)
{
  value = _value;
}

/****/

ConfigSection :: ConfigSection()
{
  name = "";
}

ConfigSection :: ConfigSection(const string &_name)
{
  name = _name;
}

ConfigSection :: ~ConfigSection()
{
  // loop and destroy entries
}

ConfigEntry * ConfigSection :: operator[] (const string &ename)
{
  vector<ConfigEntry *> :: iterator i;

  for (i = this->begin(); i != this->end(); i ++)
    if ((* i)->name == ename)
      return (* i);

  return (NULL);
}

const string ConfigSection :: get(const string &entry, const string def = "")
{
  ConfigEntry * e = (* this)[entry];

  return (e ? (const string) e->value : def);
}

void ConfigSection :: set(const string &entry, const string &value)
{
  ConfigEntry * e = (* this)[entry];

  if (e)
    e->set(value);
  else
    push_back(new ConfigEntry(entry, value));
}

void ConfigSection :: dump(FILE * fp)
{
  vector<ConfigEntry *> :: iterator i;

  for (i = this->begin(); i != this->end(); i ++) {
    ConfigEntry * e = (* i);

    if (e->value == "")
      fprintf(fp, "%s\n", e->name.c_str());
    else
      fprintf(fp, "%s = %s\n", e->name.c_str(), e->value.c_str());
  }
}

/****/

ConfigFile :: ConfigFile()
              : globalSection("Global")
{
  filename   = "";
  createMode = 0666;
  error      = false;
}

ConfigFile :: ConfigFile(string &_filename, int _createMode = 0666)
              : globalSection("Global")
{
  filename   = _filename;
  createMode = _createMode;
  error      = false;

  Load();
}

ConfigFile :: ~ConfigFile()
{
}

char * ConfigFile :: trim_string(char * str)
{
  string rv;
  char * p, * q;

  if (!str)
    return ("");

  p = str;
  q = str + strlen(str) - 1;

  while (isspace(* p))
    p ++;
  while (q >= p && isspace(* q))
    * (q--) = 0;

  return (p);
}

char * ConfigFile :: parse_section(char * str)
{
  char * q;

  if (str[0] != '[')
    return (NULL);

  q = strchr(str, ']');

  if (!q || q[1])
    return (NULL);

  * q = 0;

  return (trim_string(str + 1));
}

char * ConfigFile :: parse_value(char * str)
{
  char * q;

  q = strchr(str, '=');
  if (!q)
    return (NULL);

  * q++ = 0;

  return (trim_string(q));
}

void ConfigFile :: Load(const string _filename = "")
{
  FILE * fp;
  char buf[BUFSIZ], * s, * p;
  int line = 0;
  ConfigSection * sect = &globalSection;

  if (_filename != "")
    filename = _filename;

  if (!(fp = fopen(filename.c_str(), "r"))) {
    error = true;
    return;
  }

  while (fgets(buf, sizeof(buf), fp) != NULL) {
    line ++;
    s = trim_string(buf);

    if ((p = parse_section(s))) {
      if (!p[0])
        sect = &globalSection;
      else {
        sect = new ConfigSection(p);
        push_back(sect);
      }
    } else {
      p = parse_value(s);
      if (!p)
        p = "";

      s = trim_string(s);
      if (s[0])
        sect->set(s, p);
    }
  }

  fclose(fp);
}

void ConfigFile :: Flush()
{
  FILE * fp;

  if (!(fp = fopen(filename.c_str(), "w"))) {
    error = true;
    return;
  }

  globalSection.dump(fp);

  vector<ConfigSection *> :: iterator i;

  for (i = this->begin(); i != this->end(); i ++) {
    fprintf(fp, "[%s]\n", (* i)->name.c_str());
    (* i)->dump(fp);
  }

  fclose(fp);

  return;
}

int ConfigFile :: check_for_bool_string(const string &s)
{
  if (strcasecmp(s.c_str(), "off") == 0
      || strcasecmp(s.c_str(), "false") == 0
      || strncasecmp(s.c_str(), "no", 2) == 0)
    return (0);

  if (strcasecmp(s.c_str(), "on") == 0
      || strcasecmp(s.c_str(), "true") == 0
      || strcasecmp(s.c_str(), "yes") == 0)
    return (1);

  return (atoi(s.c_str()));
}

ConfigSection * ConfigFile :: operator[] (const string &section)
{
  vector<ConfigSection *> :: iterator i;

  for (i = this->begin(); i != this->end(); i ++)
    if ((* i)->name == section)
      return (* i);

  return (NULL);
}

const string ConfigFile :: get(const string &section, const string &entry,
                               const string def = "")
{
  ConfigSection * s = (* this)[section];
  if (!s)
    return (def);

  ConfigEntry * e = (* s)[entry];
  if (!e)
    return (globalSection.get(entry, def));
  else
    return (e->value);
}

int ConfigFile :: get(const string &section, const string &entry, int def = 0)
{
  char * _s = (char *) Alloc(128);
  sprintf(_s, "%d", def);
  string defstr = _s;
  Free(_s);

  return (check_for_bool_string(get(section, entry, defstr)));
}

#if 0
int main()
{
  string filename ="gtkaim.ini";
  ConfigFile configFile(filename);
  ConfigSection * s;
  int port;

  if (configFile.error) {
    cout << "Error!" << endl;
  } else {
    s = configFile["Connection"];
    s->dump(stdout);
  }

  configFile.Flush();

  cout << "Done." << endl;
}
#endif
