#ifndef _HAS_CONFIGFILE_H_
#define _HAS_CONFIGFILE_H_

#include <stdio.h>
#include <Memory.h>

class ConfigEntry
{
  public:
    ConfigEntry();
    ConfigEntry(const string &, const string &);
    ~ConfigEntry();

    void set(const string &);

    string name, value;
};

class ConfigSection : public vector<ConfigEntry *>
{
  public:
    ConfigSection();
    ConfigSection(const string &);
    virtual ~ConfigSection();

    ConfigEntry *operator[] (const string &);

    const string get(const string &, const string = "");
    void set(const string &, const string &);
    void set(ConfigEntry *, const string &);

    void dump(FILE *);

    string name;
};

class ConfigFile : public vector<ConfigSection *>
{
  public:
    ConfigFile();
    ConfigFile(string &, int = 0666);
    virtual ~ConfigFile();

    void Load(const string = "");
    void Flush();

    ConfigSection *operator[] (const string &);

    int get(const string &, const string &, int = 0);
    const string get(const string &, const string &, const string = "");

    ConfigSection globalSection;
    bool error;

  private:
    int createMode;
    string filename;

    int check_for_bool_string(const string &);
    char * trim_string(char *);
    char * parse_section(char *);
    char * parse_value(char *);
};

#endif /* _HAS_CONFIGFILE_H_ */
