#include <TCPThread.h>
#include <Hostinfo.h>

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <sys/time.h>
#include <netinet/in.h>

TCPThread :: TCPThread(string name_ = "untitled")
        : Thread("TCPThread: " + name_)
{
  name      = name_;

  fd        = -1;
  connected = false;
  shutdown  = false;
  server    = false;
}

TCPThread :: ~TCPThread()
{
  Close();
}

string TCPThread :: Name()
{
  return (name);
}

bool TCPThread :: Connect(string hostname, int port)
{
  struct sockaddr_in sin;
  int sin_len = sizeof(struct sockaddr_in);

  if (connected)
    Close();

  if ((fd = socket(PF_INET, SOCK_STREAM, 0)) < 0)
    return (false);

  Hostinfo hostinfo(hostname);

  sin.sin_family      = PF_INET;
  sin.sin_addr.s_addr = hostinfo.Address(0);
  sin.sin_port        = htons(port);

  if ((connect(fd, (struct sockaddr *) &sin, sin_len)) < 0)
    return (false);

  connected = true;

  Connected();

  return (true);
}

bool TCPThread :: Connect(string filename)
{
  struct sockaddr_un sun;
  int sun_len = sizeof(struct sockaddr_un);

  if (connected)
    Close();

  if ((fd = socket(PF_UNIX, SOCK_STREAM, 0)) < 0)
    return (false);

  sun.sun_family = PF_UNIX;
  strncpy(sun.sun_path, filename.c_str(), sizeof(sun.sun_path));

  if ((connect(fd, (struct sockaddr *) &sun, sun_len)) < 0)
    return (false);

  connected = true;

  Connected();

  return (true);
}

bool TCPThread :: Listen()
{
  return (false);
}

bool TCPThread :: Listen(string filename)
{
  struct sockaddr_un sun;
  int sun_len = sizeof(struct sockaddr_un);

  if ((fd = socket(PF_UNIX, SOCK_STREAM, 0)) < 0)
    return (false);

  sun.sun_family = PF_UNIX;
  strncpy(sun.sun_path, filename.c_str(), sizeof(sun.sun_path));

  if (bind(fd, (struct sockaddr *) &sun, sun_len)) {
    cout << "Unable to bind: " << strerror(errno) << endl;
    return (false);
  }

  listen(fd, 5);

  server = true;

  return (true);
}

bool TCPThread :: Pickup(int _fd)
{
  fd = _fd;

  connected = true;

  Connected();

  return (true);
}

int TCPThread :: Accept()
{
  int i;

  sem.Lock();
  i = accept(fd, NULL, 0);
  sem.Unlock();

  return (i);
}

void TCPThread :: Close()
{
  close(fd);
  fd = -1;

  connected = false;

  Disconnected();
}

void TCPThread :: Shutdown()
{
  Close();

  shutdown = true;
}

int TCPThread :: Read(void * data, int len)
{
  int rv;

  if (!connected)
    return (-1);

  if (len <= 0)
    return (-1);

  sem.Lock();
  rv = readn(fd, (byte *) data, len);
  sem.Unlock();

  if (rv < 0)
    Close();

  return (rv);
}

Buffer * TCPThread :: Read(int len)
{
  int i;
  byte * data = (byte *) Alloc(len);

  if ((i = Read(data, len)) < len) {
    Free(data);
    return (NULL);
  }

  Buffer * buffer = new Buffer(data, len);
  Free(data);

  return (buffer);
}

int TCPThread :: Send(void * data, int len)
{
  int rv;

  sem.Lock();
  rv = writen(fd, (byte *) data, len);
  sem.Unlock();

  return (rv);
}

int TCPThread :: Send(Buffer * buffer)
{
  return (Send(buffer->DataP(), buffer->Length()));
}

int TCPThread :: writen(int fd, byte * ptr, int n)
{
  int  nleft = n, nwritten;

  while(nleft > 0) {
    if((nwritten = write(fd, ptr, nleft)) <= 0)
      return (nwritten);

    nleft -= nwritten;
    ptr   += nwritten;
  }

  return n;
}

int TCPThread :: readn(int fd, byte * ptr, int n)
{
  int  nleft = n, nread;

  while(nleft > 0) {
    if((nread = read(fd, ptr, nleft)) < 0)
      return (nread);
    else if(!nread)
      break;

    nleft -= nread;
    ptr   += nread;
  }

  return (n - nleft);
}

void TCPThread :: Main()
{
  struct timeval timeout;
  fd_set rfds;
  int maxfd, x;

  timeout.tv_sec  = 1;
  timeout.tv_usec = 0;
  maxfd           = fd + 1;

  while (!shutdown) {
    if (!server && !connected) {
      // sleep(2);
      continue;
    }

    FD_ZERO(&rfds);
    FD_SET(fd, &rfds);

    if ((x = select(maxfd, &rfds, NULL, NULL, &timeout)) == -1) {
      Shutdown();
    } else 
    if (x == 0) {
      // timeout
    } else {
      if (server)
        ClientWaiting();
      else
        DataWaiting();
    }
  }
}

void TCPThread :: DataWaiting()
{
}

void TCPThread :: ClientWaiting()
{
}

void TCPThread :: Connected()
{
}

void TCPThread :: Disconnected()
{
}


