#ifndef _HAS_TLV_H_
#define _HAS_TLV_H_

#include <vector>

#include <utils.h>
#include <Memory.h>
#include <Buffer.h>

#include <netinet/in.h>

class TLVElement {
  public:
    TLVElement(Buffer *);
    TLVElement(Int16, Int16);
    TLVElement(Int16, Int16, byte *);
    TLVElement(const TLVElement &tlvElement);
    ~TLVElement();

    Int16  Type() const;
    Int16  Length() const;
    byte * Value() const;

    int Write(Buffer *);

    Int16  type;
    Int16  length;
    byte * value;
};

class TLV
{
  public:
    TLV(bool = true);
    TLV(Buffer *, bool = true);
    TLV(Buffer *, int, bool = true);
    ~TLV();

    void Append(TLVElement &);
    int Write(Buffer *);
    int Read(Buffer *);
    int Read(Buffer *, int);
    void IndexMode(bool);

    int Put(Int16 type);

    int PutUInt8(Int16, UInt8);
    UInt8 GetUInt8(Int16);

    int PutInt8(Int16, Int8);
    Int8 GetInt8(Int16);

    int PutUInt16(Int16, UInt16);
    UInt16 GetUInt16(Int16);

    int PutInt16(Int16, Int16);
    Int16 GetInt16(Int16);

    int PutUInt32(Int16, UInt32);
    UInt32 GetUInt32(Int16);

    int PutInt32(Int16, Int32);
    Int32 GetInt32(Int16);

    int PutRaw(Int16, Int16, byte *);
    byte * GetRaw(Int16);

    int PutBuffer(Int16, Buffer *);
    Buffer * GetBuffer(Int16 type);

    int PutString(Int16, string);
    string GetString(Int16);

    TLVElement * IsPresent(Int16);
    int Length();
    int Length(Int16);

    void test();

  private:
    bool indexmode;
    bool swap;
    vector<TLVElement> elements;
};

#endif /* _HAS_TLV_H_ */
